package redis

import (
	"context"
	"crypto/tls"
	"fmt"
	"sync"

	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/otel"
	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/trace"

	redis "github.com/redis/go-redis/v9"
	"github.com/rs/zerolog"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/ambient/config"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather/recorder/recorders"
)

const (
	DEF_RETAIN  = 120
	UPDATES_KEY = "weatherUpdates"
)

type RedisRecorder struct {
	baseCtx context.Context
	tracer  trace.Tracer
	redis   *redis.Client
	config  *config.AmbientLocalExporterConfig
	log     *zerolog.Logger
	appKey  string // prefix for redis keys, uses app name, environment, and version
	keep    int
	*sync.RWMutex
}

func (r *RedisRecorder) Init(ctx context.Context, opts *recorders.RecorderOpts) {
	if opts.RetainLast < 1 {
		opts.RetainLast = DEF_RETAIN
	}

	r.log = zerolog.Ctx(opts.BaseCtx)

	r.tracer = otel.GetTracer(r.baseCtx, "redisRecorder")
	ctx, span := r.tracer.Start(ctx, "redisRecorder.init", trace.WithAttributes(
		attribute.String("redisHost", opts.AppConfig.RecorderConfig.RedisConfig.RedisHost),
		attribute.Int("retainLast", opts.RetainLast),
		attribute.Int("redisPort", opts.AppConfig.RecorderConfig.RedisConfig.RedisPort),
		attribute.Bool("tls", opts.AppConfig.RecorderConfig.RedisConfig.RedisTLS),
	))
	defer span.End()

	r.config = opts.AppConfig
	r.keep = opts.RetainLast
	r.baseCtx = opts.BaseCtx
	r.RWMutex = &sync.RWMutex{}

	// Unique key prefix for this version/env/name of exporter
	// will be consistent across replicas, but resets on upgrade
	// as it is using version
	r.appKey = util.GetAppHash(r.config.AppConfig)

	r.MustInitRedis(ctx)
}

func (r *RedisRecorder) MustInitRedis(ctx context.Context) {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.init.redis")
	defer span.End()

	rc := r.config.RecorderConfig.RedisConfig

	var tlsConfig *tls.Config
	if rc.RedisTLS {
		tlsConfig = &tls.Config{
			ServerName:         rc.RedisHost,
			InsecureSkipVerify: rc.RedisTLSInsecure,
		}
	}

	r.redis = redis.NewClient(&redis.Options{
		Addr:       fmt.Sprintf("%s:%d", rc.RedisHost, rc.RedisPort),
		ClientName: fmt.Sprintf("%s-%s", r.config.Name, r.config.Environment),
		Username:   rc.RedisUser,
		Password:   rc.RedisPassword,
		DB:         rc.RedisDB,
		TLSConfig:  tlsConfig,
	})

	span.AddEvent("redis client ready")

	resp := r.redis.Ping(ctx)
	if resp.Err() != nil {
		span.RecordError(resp.Err())
		span.SetStatus(codes.Error, resp.Err().Error())
		r.log.Fatal().Err(resp.Err()).Msg("failed to ping redis")
	}

	span.AddEvent("redis client ping ok")
	span.SetStatus(codes.Ok, "")

	r.log.Info().Str("appKey", r.appKey).
		Msg("redis ping ok, client ready")
}

func (r *RedisRecorder) Key() string {
	return fmt.Sprintf("%s:%s", r.appKey, UPDATES_KEY)
}
