package redis

import (
	"context"
	"encoding/json"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *RedisRecorder) Set(ctx context.Context, u *weather.WeatherUpdate) error {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.set", trace.WithAttributes(
		attribute.String("stationName", u.GetStationName()),
		attribute.String("stationType", util.DerefStr(u.StationType)),
	))
	defer span.End()

	r.Lock()
	defer r.RUnlock()

	// First ensure we can prepare our payload
	data, err := json.Marshal(u)
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())

		r.log.Err(err).Send()
		return err
	}

	return r.set(ctx, data)
}

func (r *RedisRecorder) set(ctx context.Context, data []byte) error {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.set.push", trace.WithAttributes(
		attribute.Int("updateBytes", len(data)),
	))
	defer span.End()

	// Atomic, push and trim
	tx := r.redis.TxPipeline()
	tx.LPush(ctx, r.Key(), data)
	tx.LTrim(ctx, r.Key(), 0, int64(r.keep)-1)

	if rErr, err := tx.Exec(ctx); err != nil {
		for _, cmd := range rErr {
			span.RecordError(cmd.Err())
		}
		span.SetStatus(codes.Error, err.Error())
		r.log.Err(err).Send()
		return err
	}

	span.SetAttributes(attribute.Int("updateCount", r.count(ctx)))
	span.SetStatus(codes.Ok, "")
	return nil
}
